/*** functions.js *************************************************************
*** Athur:        Henning Kristiansen                                       ***
*** Program:      Map Editor                                                ***
*** Date:         2003-05-11                                                ***
*** Project:      OneMap.Submission.GUI                                     ***
***                                                                         ***
*** Description:                                                            ***
***   This is the main file for the Map Editor. Everything exeptc click     ***
***   events in the script goes through here so this is the core of the     ***
***   editor.                                                               ***
***                                                                         ***
*** THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED        ***
*** WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF    ***
*** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN ***
*** NO EVENT SHALL THE ONE MAP PROJECT OR ITS CONTRIBUTORS BE LIABLE FOR    ***
*** ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL  ***
*** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS ***
*** OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)   ***
*** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,     ***
*** STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING   ***
*** IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE      ***
*** POSSIBILITY OF SUCH DAMAGE.                                             ***
******************************************************************************/

/**
	The coordinates for where the mouse is on the map
**/
var startX, startY, x, y, lastX, lastY, stopX, stopY;

/**
	The coordinates for where the mouse is in the window
**/
var lastClientX, lastClientY, currentClientX, currentClientY;
/**
	This will be how width and heigh the map is per pixel
**/
var pixSizeX, pixSizeY;
/**
	This is the orginal pixSize without the plugins is beeing zoomed.
	This will allways be the same as beforce since the editor
	sets all zooms in the plugin to be the orginal zoom
**/
var origPixSizeX, origPixSizeY

/**
	This is how width and heigh the map view is
**/
var pixWidth, pixHight;
/**
	Booleans for if the user is moving a point og draging for points
**/
var draging = false, moving = false;
/**
	svgdoc is the main SVG doument that is the root of the DOM three
	svgobj is just a temp variable
	svgPoints is the root of the selected points
	svgSVGObj is the edior object
**/
var svgdoc, svgobj, svgPoints, svgSVGObj;

/**
	box_drag is the objekct for the dragbox rectangle
	console is the pointer to the SVG Console window
**/
var box_drag, console;
/**
	These are just variables that is a pointer to the text objects
**/
var text_x, text_y, text_info;
/**
	The upperleft corner for the map at all time
**/
var viewX, viewY;
/**
	How much have the user moved the points in each direction and what
	is the starting point
**/
var moveX = 0, moveY = 0, startMoveX, startMoveY;
/**
	How meny points is selected
**/
var marked = 0;
/**
	Pointer to the root of the map
**/
var map;
/**
	Right mouseclicks will also be treated as a mouseclick and lot's of bad 
	things can happens if we don't stop the code when this occurs
**/
var rightButtonJustClicked = false;
/**
	The last info tekst so it can go back to what it was before the change.
**/
var lastInfoText;
/**
	The actual size of the svg window
**/
var windowHeight, windowWidth
/**
	mouseClickCount is to count how meny clicks the user has pressed.
	mouseClickTarget is what was clicked
**/
var mouseClickCount = 0, mouseClickTarget;
/**
	The ratio of how much the view has been changes since last resize
**/
var ratioHeight, ratioWidth;
/**
	Variables used for the zoom, scroll and reset function
**/
var doNothingZoom;
var zoomNumber = 0;
/**
	This will contain the coverall object in the DOM
**/
var coverall;
/***init()*********************************************************************
*** This is the first thing that happens when the uses open the SVG editor. ***
*** What this do is that it collects all the variables that it's going to   ***
*** use very often and some other pre-maintain things                       ***
******************************************************************************/
function init( evt ) {	
	svgdoc = evt.getTarget().getOwnerDocument();
	svgSVGObj = svgdoc.getDocumentElement();
	map = svgdoc.getElementById('map');

	text_info = svgdoc.getElementById('info');
	text_x = svgdoc.getElementById('x');
	text_y = svgdoc.getElementById('y');

	svgPoints = svgdoc.getElementById('points');
	box_drag = svgdoc.getElementById('dragbox');
	console = svgdoc.getElementById('console');
	infoMenu = svgdoc.getElementById('InfoMenu');
	coverall = svgdoc.getElementById('coverall');
	map = svgdoc.getElementById('map');

	ratioWidth = window.innerWidth/800;
	ratioHeight = window.innerHeight/500;
	
	setCoverallData();
	setWindowScale();
	
	/**
		Removes any text object that has been automaticly generated and that 
		is not welcome
	**/
	removeAllTextFromSVG(map);
	removeAllTextFromSVG(svgPoints);

	/**
		Initiates all of the other modules
	**/
	
	initFeature();
	initHistory();
	initNavigator();
	initLayer();
	
	/**
		resets the coordiantes of the map to be in the way the parser gets 
		and resize the width and height to be as the browser is.
	**/

	var viewbox = new String(svgSVGObj.getAttribute("viewBox"));
	var coordinates = viewbox.split(' ');		// x,y,width,height
	
	resetOrginalCoords(
		parseFloat(coordinates[0]),
		parseFloat(coordinates[1]),
		parseFloat(coordinates[2])*ratioWidth,
		parseFloat(coordinates[3])*ratioHeight,
		false
	);
	resetCoords(false);
	
	/**
		This is to scale the navigator window for what it is now. This needed to
		be done last since it is depended that some variables has been
		calculated
	**/	
	setNavigatorWindow();
}


/***resetCoords()**************************************************************
*** This is the function that is called when the user zooms and scroll the  ***
*** SVG and when the user resize the browser                                ***
***                                                                         ***
*** This function resets the editor to think that the new change is the     ***
*** origional view for the editor. This is a smart thing to do because then ***
*** you do not have to think about if the view is 'not where it could be'   ***
*** when you calculate the x and y for the mouse that are very important    ***
*** values the editor uses whitch this function sets                        ***
***                                                                         ***
*** The zoom parameter is there because when the user zooms with the        ***
*** buld-in function in the plugin this function gets called to times! And  ***
*** we do not want this to do the same thing two times so this stops the    ***
*** second time                                                             ***
***                                                                         ***
*** This function also calls when the javascipt some functions like         ***
***   translate.setX(0);                                                    ***
***   translate.setY(0);                                                    ***
***   svgSVGObj.setCurrentScale(1);                                         ***
***                                                                         ***
*** So the doNothingZoom variable is set right beforce this function is set ***
*** and the start of this function pics this up and return the function     ***
*** without doing anything                                                  ***
******************************************************************************/

function resetCoords(zoom) {
	if (doNothingZoom) return;
	if (zoom) zoomNumber++;
	if (zoomNumber == 2) {
		zoomNumber = 0;
		return;
	}
	/**
		Stops draging for points if this accurs
	**/
	draging = false;
	
	/**
		Scale value is a float value that represent how meny percent of the acctual
		view is the current view. This will be used to calculate the next view.
	**/	
	var scale = svgSVGObj.getCurrentScale();

	/**
		translate will be were how long from the acctual upper left x and y value is
		from the current view.
	**/	
	var translate = svgSVGObj.getCurrentTranslate();

	/**
		origPixSize? is the how meny pixuals from the map coordinates there is per 
		pixuals on the screen. this will	calculate what the new pixuals per 
		screenpixual is after the zoom.
	**/
	pixSizeX = origPixSizeX / scale;
	pixSizeY = origPixSizeY / scale;
	
	/**
		This calculate the new viewX and viewY afther it's been moved. translate.getX() 
		gets the value of how far it is from the orginal view. The viewX will be sett 
		as the new orginal view later
	**/
	
	viewX -= (translate.getX() * pixSizeX);
	viewY -= (translate.getY() * pixSizeY);

	/**
		This is how much the window has been resized in percent
	**/
	
	ratioWidth = window.innerWidth/windowWidth;
	ratioHeight = window.innerHeight/windowHeight;
	
	/**
		This function calculate if you have resized the window wider then
		the 800/500 or taller and uses this to scale the console. Se the
		function to find out more of how it works
	**/
	
	setWindowScale();

	/**
		The next functions is just saying to the console that you could reshape
		and move your menues		
	**/
	scaleHistoryMenu(true);
	scaleNavigatorMenu(true);
	scaleFeatureMenu(true);
	scaleLayerMenu(true);

	var infoMenuHeight = window.innerWidth*(20/800);
	infoMenu.setAttribute('y', (window.innerHeight-infoMenuHeight) + "");
	infoMenu.setAttribute('height', infoMenuHeight + "");

	/**
		Since the points and stroke sizes are in percent of the view this have 
		to	calculate the new percent to the new resized view.
	**/	
	setHighlightedFutureStrokeWidth();
	resetAllPointRadius();
		
	var stroke = (0.4 - (0.2 * windowscale)) + "%";
	map.setAttribute("stroke-width", stroke );
	
	/**
		It is here that we reset the map to use the new coordinates
		See the resetOrginalCoords() function for more information
	**/	
	var newScale = 1 / parseFloat(scale);
	
	viewWidth *= (newScale*ratioWidth);
	viewHeight *= (newScale*ratioHeight);
	
	resetOrginalCoords(viewX,viewY,viewWidth,viewHeight,false);
	
	/**
		This moves the upperleft corner of the console to the new corner of 
		the view
	**/
	var newtransform = "translate(" + viewX + " " + viewY + ")";	
	console.setAttribute('transform', newtransform);

	/**
		The next commands is resetting the plugin to use the new values as 
		the actual values. It set the doNothingZoom to true because all of 
		these functions makes a new event that calls this resetCoords function. 
		and stops if it's true. If we didn't stop this would go on and on 
		endlessly
	**/
	doNothingZoom = true;
	svgSVGObj.setCurrentScale(1);
	translate.setX(0);
	translate.setY(0);
	doNothingZoom = false;
	
	/*
		Next is to set the neavigator view to were this new view is. To see 
		more about how it works go to the function in navigator.js	
	*/
		
	setNavigatorView();
	/**
		Sets the zoomnumber to be 0 now at the end. So the next time it will 
		start all over again.
	**/
	zoomNumber = 0;
}

/***resetOrginalCoords()*******************************************************
*** This function is here just to calculate and resets the coordiantes for  ***
*** the map. This is called after a zoom, resize or scroll and even after   ***
*** the user moves the navigator view with it mouse.                        ***
***                                                                         ***
*** The x y w h parameters the new view and the trans pareamter is a        ***
*** boolean that is if this function could move the console                 ***
******************************************************************************/
function resetOrginalCoords(x,y,w,h,trans) {	
	/**
		Sets the global variables for what the new view is
	**/
	viewX = x; viewY = y, viewWidth = w, viewHeight = h;

	/**
		The next one sets the new view. This calls the resetCoords because but we
		do not wan't it to do that so we set the doNothingZoom to be true
	**/
	doNothingZoom = true;
	var newViewBox = x + " " + y + " " + w + " " + h;
	svgSVGObj.setAttribute("viewBox",newViewBox);
	doNothingZoom = false;
	
	/**
		Sets the width and height for the console to be the same as the width
		of what's being seen
	**/
	var svgConsole = svgdoc.getElementById('svgConsole');	
	svgConsole.setAttribute("width", w + "");
	svgConsole.setAttribute("height", h + "");
	
	/**
		Sets the new SVG window size. It's better to use this because then we
		will know what the width was before any new resize and then calcualate
		how much the change is.
	**/
	windowWidth = window.innerWidth;
	windowHeight = window.innerHeight;
	
	/**
		Resets the console viewbox to be the actual pixuals of the screen
	**/
	var viewBoxConsole = "0 0 " + windowWidth + " " + windowHeight;
	svgConsole.setAttribute("viewBox", viewBoxConsole);
	
	/**
		Set the new orginial pix size that is used to calculate what the
		changes are on the next zoom
	**/
	origPixSizeX = viewWidth / windowWidth;
	origPixSizeY = viewHeight / windowHeight;
	
	/**
		This is if the function could move the console
	**/
	if (trans) {
		/**
			Moves the console and calculate new values
		**/		
		var newtransform = "translate(" + x + " " + y + ")";
		console.setAttribute('transform', newtransform);
		setNavigatorView();
		var scale = svgSVGObj.getCurrentScale();
		var translate = svgSVGObj.getCurrentTranslate();
		pixSizeX = origPixSizeX / scale;
		pixSizeY = origPixSizeY / scale;
	
		viewX -= translate.getX() * pixSizeX;
		viewY -=	 translate.getY() * pixSizeY;			
	}	
}
/***setCoverallData()**********************************************************
*** This sets the coverall data that will be used later                     ***
******************************************************************************/
function setCoverallData() {
	coverallX = parseFloat(coverall.getAttribute('x'));
	coverallY = parseFloat(coverall.getAttribute('y'));
	coverallWidth = parseFloat(coverall.getAttribute('width'));
	coverallHeight = parseFloat(coverall.getAttribute('height'));	
	coverallX2 = coverallX + coverallWidth;
	coverallY2 = coverallY + coverallHeight;
}
/***pointInsideCoverall()******************************************************
*** Returnts true or false if the given x and y is inside the coverall      ***
******************************************************************************/
function pointInsideCoverall(checkX, checkY) {
	if ((coverallX > checkX) || (checkX > coverallX2))
		return false;
	if ((coverallY > checkY) || (checkY > coverallY2))
		return false;
	
	return true;
}
/***removeAllTextFromSVG()*****************************************************
*** This is a recusive function that removes all of the unwanted text nodes ***
*** that the SVG Dom automaticly has created                                ***
******************************************************************************/
function removeAllTextFromSVG(root) {
	var child = root.childNodes;
	for(var i=0;i<child.length;i++) {
		if (child.item(i).nodeName == '#text') {
			root.removeChild(child.item(i));
			i--;
		}
		else
			removeAllTextFromSVG(child.item(i));
	}
}
/***info()*********************************************************************
*** A simple function that sets the text at the info textnode               ***
******************************************************************************/

function info( text ) {
	lastInfoText = text_info.getFirstChild().getData();
	text_info.getFirstChild().setData(text);
}
/***info()*********************************************************************
*** Sets the last used infotext                                             ***
******************************************************************************/
function setLastInfoTekst() {
	text_info.getFirstChild().setData(lastInfoText);
}

/***trim()*********************************************************************
*** Since there is noe trim function in javascript I had to make one        ***
******************************************************************************/
function trim(str) {
	return str.replace(/^\s+/,'').replace(/\s+$/,'');
}

/***startMove()****************************************************************
*** This is started when the user clicks on a selected point. The first     ***
*** it does is to set were it started so it's easy to go back               ***
******************************************************************************/
function startMove(evt) {
	lastX = x;
	lastY = y;	
	startMoveX = x;
	startMoveY = y;
	moving = true;
}

/***stopMove()*****************************************************************
*** This is when the user stops clicking after having clicked on a selected ***
*** point. If there is more then 5 marked at a time this will also move the ***
*** map because it has not been moved.                                      ***
***                                                                         ***
*** After the move this function will add this to the history               ***
******************************************************************************/
function stopMove(evt) {
	if ((x == startMoveX) && (y == startMoveY))
		return;
	moving = false;
	if (marked > 5) {
		moveX = x - startMoveX;
		moveY = y - startMoveY;
		moveSelectedPoints(true, false);
	}
	else
		moveSelectedPoints(true, true);
	addMoveToHistory();
}

/***stopMove()*****************************************************************
*** This is when the mouse has been clicked on a selected point and the     ***
*** is moving the mouse with the mousebutton pressed in. This will only     ***
*** the points if there is over 5 points selected at a time                 ***
******************************************************************************/
function move( evt, what) {
	text_x.getFirstChild().setData('x = ' + x);
	text_y.getFirstChild().setData('y = ' + y);
	if (moving) {
		moveX = x - lastX;
		moveY = y - lastY;
		if (marked > 5) moveSelectedPoints(false, true);
		else moveSelectedPoints(true, true);
		lastX = x;
		lastY = y;
	}
}
/***moveSelectedPoints()*******************************************************
*** This is were the acctual move happens                                   ***
***                                                                         ***
*** The parameter speaks for itself                                         ***
******************************************************************************/
function moveSelectedPoints(moveMap, movePoints) {
	var pointstr = ""
	
	/**
		Goes trough every line in the map that has been selected
	**/
	var pol = svgPoints.childNodes;
	for(var i=0;i<pol.length;i++) {
		/**
			Finds the object for the line and gets out it points and make the 
			points into an array	if the moveMap is true
		**/
		var svgobj = svgdoc.getElementById(pol.item(i).getAttribute('source'));
		if (moveMap) {
			pointstr = new String(svgobj.getAttribute('points'));
			var points = trim(pointstr).split(" ");
		}
		
		/**
			Goes throuh every points on this specific line that has been 
			selected and finds the x position of that point. After that it 
			will move the point if the movePoints are	true and it will 
			change the value of the array to the new move if moveMap is true
		**/
		var kid = pol.item(i).childNodes;
		for(var j=0;j<kid.length;j++) {
			var pos = parseFloat(kid.item(j).getAttribute('sourceposition'));
			
			if (moveMap) {
				points[pos] = parseFloat(points[pos]) + moveX;
				points[pos+1] = parseFloat(points[pos+1]) + moveY;
			}
			if (movePoints) {
				kid.item(j).setAttribute("cx", (parseFloat(kid.item(j).getAttribute("cx")) + moveX) + "");
				kid.item(j).setAttribute("cy", (parseFloat(kid.item(j).getAttribute("cy")) + moveY) + "");
			}
		}
		/** 
			If the map could move this will change the points attribute to 
			it's new value with the changes that is made.
		**/
		if (moveMap) {
			var p = '';
			for(j=0;j<points.length;j++) 
				p = p + points[j] + ' ';
			svgobj.setAttribute('points', p);			
		}
	}
}

/***checkKeyPress()************************************************************
*** cheks what key is been pressed                                          ***
******************************************************************************/
function checkKeyPress(evt) {
	/**
		If the ESC is pressed it will stop the draging and moving of menu items
	**/
	if (evt.getCharCode() == 27) {
		var svgstyle = box_drag.getStyle();
		svgstyle.setProperty ('visibility', 'hidden');
		draging = false;	
		dragHistoryMenu = false;
		dragNavigatorMenu = false;
		dragFeatureMenu = false;
		dragLayerMenu = false;
	}	
}

/***startClick()***************************************************************
*** This is the function that every mouseclick starts with. The what        ***
*** parameter desides what could be done.                                   ***
***                                                                         ***
*** If will stop everything if the right button is clicked or has just been ***
*** clicekd                                                                 ***
******************************************************************************/
function startClick(evt,what) {
	if (evt.button==2) {
		rightButtonJustClicked = true;
		return;
	}
	if (rightButtonJustClicked) {
		rightButtonJustClicked = false;
		return;
	}	
	if (evt.altKey==true)
		return;
	if (evt.ctrlKey==true)
		return;
	if (what == 'back')
		return;	
	if (what == 'regular') 
		startDrag(evt);	
	if (what == 'selected') 
		startMove(evt);
	if (what == 'history') 
		startClickHistory(evt);
	if (what == 'layer')
		startClickLayer(evt);
	if (what == 'navigator') 
		startClickNavigator(evt);
	if (what == 'feature')
		startClickFeature(evt);
}

/***stopClick()****************************************************************
*** This is the function that happens every time the mouse stoppes beeing   ***
*** clicked. It's the what paramter desides what could be done.             ***
***                                                                         ***
*** If it's moving then it will stop move no mather what and where it is    ***
******************************************************************************/
function stopClick(evt,what) {
	if (moving) {
		moving = false;
		stopMove(evt);
		return;
	}	
	if (what == 'selected') {
		stopMove(evt);
		return;
	}
	if (what == 'history')
		stopClickHistory(evt);
	if (what == 'layer')
		stopClickLayer(evt);
	if (what == 'feature')
		stopClickFeature(evt);	
	if (what == 'navigator')
		stopClickNavigator(evt);
	if (draging) 
		stopDrag(evt);
}

/***findPointsByClick()********************************************************
*** This is if the javascript detects only a simple mouseclick and this     ***
*** look for a point right next to the mouse cursor                         ***
******************************************************************************/
function findPointsByClick(evt) {
	startX = x - (viewWidth * 0.007);
	startY = y + (viewHeight * 0.007);
	
	stopX = x + (viewWidth * 0.015);
	stopY = y - (viewHeight * 0.015);
	lowY = 999999;
	highY = 0;
	lowX = 99999
	highX = 0;
	
	findPoints(map);
	
}

/***startDrag()****************************************************************
*** This is the start of draging after points. If the shift key is pressed  ***
*** it will look for more points if not it will remove all the others       ***
******************************************************************************/
function startDrag(evt) {
	draging = true;
	if (evt.shiftKey!=true) {
		var kid = svgPoints.childNodes;
		
		while (kid.length>0)
			svgPoints.removeChild(kid.item(0));
	}
	startX = x;
	startY = y;
	
	box_drag.setAttribute ('x', startX + "");
	box_drag.setAttribute ('y', startY + "");
	box_drag.setAttribute ('width', 0);
	box_drag.setAttribute ('height', 0);
	
	var svgstyle = box_drag.getStyle();
	svgstyle.setProperty ('visibility', 'visible');	
}

/***drag()*********************************************************************
*** This just creates a rectangular of the dragbox. This will be shown no   ***
*** the right way no mather what way you drag                               ***
******************************************************************************/
function drag() {
	box_drag.setAttribute ('x', Math.min(x,startX) + "");	
	box_drag.setAttribute ('width', Math.abs(startX-x) + "");

	box_drag.setAttribute ('y', Math.min(y,startY) + ""); 
	box_drag.setAttribute ('height', Math.abs(startY-y) + "");
}

/***stopDrag()*****************************************************************
*** This is when the user stops clicking the mouse after a drag. Here it    ***
*** will look for points to selected. If the shitf key is pressed it will   ***
*** not remove the other selected points. Then it set what the start and    ***
*** stop vaules are no mather what way the box has been dragged.            ***
***                                                                         ***
*** It stoppes if it hasent moved anything                                  ***
******************************************************************************/
function stopDrag(evt) {
	var svgstyle = box_drag.getStyle();
	svgstyle.setProperty ('visibility', 'hidden');
	draging = false;	

	if ((x == startX) && (y == startY))
		return;
	if (evt.shiftKey!=true) {	
		removeAllChilds(svgPoints);
		marked = 0;
	}	

	startX = Math.min(startX,x);
	startY = Math.max(startY,y);
	stopX = startX + parseFloat(box_drag.getAttribute('width'));
	stopY = startY - parseFloat(box_drag.getAttribute('height'));
	
	findPoints(map);	
}

/***findPoints()***************************************************************
*** Here it will traverse it selv trough every line in the SVG editor       ***
*** looking for points inside the dragbox.                                  ***
***                                                                         ***
*** This is a recusive function that stops when it comes to a object that   ***
*** is not a <g> tag. When it finds anything else it checks the line with   ***
*** checkPoly() function'                                                   ***
***                                                                         ***
*** If it comes to a <g> tag it will check if that tag is editable or       ***
*** hidden. And if it is it will jump over this element                     ***
******************************************************************************/

function findPoints(root) {
	var kid = root.childNodes;
	for(var i=0;i<kid.length;i++) {
		if (kid.item(i).nodeName != "g")
			checkPoly(kid.item(i));
		else {
			if (kid.item(i).getAttribute('editable') == "false")
				continue;
			if (kid.item(i).getAttribute('visibility') == "hidden")
				continue;				
			findPoints(kid.item(i));
		}
	}
}
/***checkPoly()****************************************************************
*** The first thing it does is to split the points attribute in to a array  ***
*** and then it gooes through the array and checks if the x and y are       ***
*** inside the dragbox. If that it creates a marked point of it             ***
******************************************************************************/
var lowY = 99999, highY = 0;
var lowX = 99999, highX = 0;
function checkPoly(p) {	
	var pointstr = new String(p.getAttribute('points'));
	var points = trim(pointstr).split(" ");
	for(var i = 0;i<points.length;i++) {
		if (parseFloat(points[i]) > highX)
			highX = parseFloat(points[i]);
		if (parseFloat(points[i]) < lowX)
			lowX = parseFloat(points[i]);
		if (parseFloat(points[i+1]) > highY)
			highY = parseFloat(points[i+1]);
		if (parseFloat(points[i+1]) < lowY)
			lowY = parseFloat(points[i+1]);
		if (checkPoint(points[i], points[i+1])) {
			createMarked(points[i], points[i+1], p.getAttribute('id'),i, p.getAttribute('layer'), p.getAttribute('edited'));
		}
		i++;
	}
}
/***checkPoint()***************************************************************
*** Returns a boolean of if the checkX and checkY are inside the dragbox    ***
******************************************************************************/
function checkPoint(checkX, checkY) {	
	if (startX > checkX) return false;
	if (stopX < checkX) return false;

	if (startY < checkY) return false;
	if (stopY > checkY) return false;

	return true;
}

/***createMarked()*************************************************************
*** This adds a circle to the SVG right on top of where it has found a      ***
*** point. It collects all of the points that is in the same line in a new  ***
*** objectlist (<g>-tag). With every point created it saves the value of    ***
*** whitch line it's from and what position in the point-list it is. The x  ***
*** and y value will be the center of the ring                              ***
***                                                                         ***
*** It also checks if the point has been added before and stops if it is    ***
******************************************************************************/
function createMarked( x, y, name, position, layer, edited) {	
	var check = svgdoc.getElementById('marked' + name + position);
	if (check != null) 
		return;
	marked++;
	var pol = svgdoc.getElementById('marked' + name);
	
	if (pol == null) {
		pol = svgdoc.createElement("g");
		pol.setAttribute("id",'marked' + name);
		pol.setAttribute("source", name);
		pol.setAttribute("layer", layer);
		pol.setAttribute("edited", edited);
		svgPoints.appendChild(pol);
	}
	
	var shape = svgdoc.createElement("circle");
	shape.setAttribute("id", 'marked' + name + position);
	shape.setAttribute("cx", x + "");
	shape.setAttribute("cy", y + "");
	var radius = 0.006 * (1 - (viewWidth/coverallWidth));
	setPointsRadius(shape);
	shape.setAttribute("sourceposition", position);
	pol.appendChild(shape);
	
}
/***findAndRemovePointsWithThisLayer()*****************************************
*** This finds and remove all of the points that are asign to a layer.      ***
*** Mainly used when a user removes editable or visiblitly at a layer       ***
******************************************************************************/
function findAndRemovePointsWithThisLayer(layername) {
	var kid = svgPoints.childNodes;
	
	for(var i=0;i<kid.length;i++)
		if ("Map-" + kid.item(i).getAttribute('layer') == layername) {
			svgPoints.removeChild(kid.item(i));
			i--;
		}
}

/***resetAllPointRadius()******************************************************
*** This goes throuh all of the selected points and run the setPointsRadius ***
*** function on each selected point.                                        ***
******************************************************************************/
function resetAllPointRadius() {
	var kid = svgPoints.childNodes;
	
	for(var i=0;i<kid.length;i++)
		setPointsRadius(kid.item(i));	
}
/***setPointsRadius()**********************************************************
*** This resets the point radius to a view that best fits the new resized   ***
*** window                                                                  ***
******************************************************************************/
function setPointsRadius(shape) {	
	var radius = (3 - (1.5 * windowscale)) + "%";
	shape.setAttribute("r", radius);
}
/***checkClickPoint()**********************************************************
*** This is called when you just click on a selected point. If you have     ***
*** presed shifkey while clicking it will remove the point                  ***
***                                                                         ***
*** If not it will check if the user has clicked twiced within 300ms. Since ***
*** javascript does not have a doubleclick event I had to make my own. If   ***
*** it has regisered two clicks within the 300ms it will open the feature   ***
*** info menu                                                               ***
******************************************************************************/
function checkClickPoint(evt) {
	if (evt.shiftKey) {
		var parent = evt.getTarget().parentNode;
		parent.removeChild(evt.getTarget());
		marked--;
		mouseClickCount = 0;
		mouseClickTarget = null;
		return;
	}
	mouseClickCount++;
	mouseClickTarget = evt.getTarget();
	clickEVT = evt;
	if (mouseClickCount == 1) 
		var interval = window.setTimeout("removeClickInfo()",250);
	else 
		getFeautreInfo(mouseClickTarget);
}
/***removeClickInfo()**********************************************************
*** This is called 300ms after the user has clicked on a point to remove    ***
*** the information gathered 300ms ago                                      ***
******************************************************************************/
function removeClickInfo() {
	mouseClickCount = 0;
	mouseClickTarget = null;	
}

/***removeAllChilds()**********************************************************
*** This removes all the childs of an object                                ***
******************************************************************************/
function removeAllChilds(p) {
	var kid = p.childNodes;		
	while (kid.length>0)
		p.removeChild(kid.item(0));
}
/***checkmouse()***************************************************************
*** This is the function that is called every time the user moves the mouse ***
*** inside the SVG window. It updaets the x and y attribute and changes the ***
*** x and y text nodes so the user can see where he/she is                  ***
******************************************************************************/
function checkmouse( evt,what ) {
	if (what == 'back')
		return;	
	lastClientX = currentClientX;
	lastClientY = currentClientY;


	currentClientX = evt.getClientX();
	currentClientY = evt.getClientY();

	x = viewX + (currentClientX * pixSizeX);
	y = (viewY + (currentClientY * pixSizeY)) * -1;
	
	/**
		Every item at the menu has it owns mousecheck so it goes through there
	**/
	checkMouseFeature(evt);
	checkMouseHistory(evt);
	checkMouseLayer(evt);
	checkMouseNavigator(evt);

	if (what != 'regular')
		if (!pointInsideCoverall(x,y))
			return;

	if (moving) {
		move(evt);
		return;
	}
				
	text_x.getFirstChild().setData('x = ' + x);
	text_y.getFirstChild().setData('y = ' + y);

	if (draging)
		drag();
}