/*** layer.js *****************************************************************
*** Athur:        Henning Kristiansen                                       ***
*** Program:      Map Editor                                                ***
*** Date:         2003-05-11                                                ***
*** Project:      OneMap.Submission.GUI                                     ***
***                                                                         ***
*** Description:                                                            ***
***   This file contains all of the function regared the layer menu.        ***
***   It does not meen that it a stand-alone module because this is         ***
***   depended on variables from other files in this prosject. And is also  ***
***   depended on the way the SVG file is structured and has all the ID's   ***
***   that this looks for. And that noone of these ID's is being used more  ***
***   than that one time                                                    ***
***                                                                         ***
***   Every function goes throught the functions.js file except the once    ***
***   that is onclick events from the SVG                                   ***
***                                                                         ***
*** THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED        ***
*** WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF    ***
*** MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN ***
*** NO EVENT SHALL THE ONE MAP PROJECT OR ITS CONTRIBUTORS BE LIABLE FOR    ***
*** ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL  ***
*** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS ***
*** OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)   ***
*** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,     ***
*** STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING   ***
*** IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE      ***
*** POSSIBILITY OF SUCH DAMAGE.                                             ***
******************************************************************************/
/**
	All the variables needed within the layer
**/
var layer, layerMenu, layerLength = 0, layerPosition = 0, layerButtons, layerScrollbar, layerScrollbarY, layerScrollbarHeight;
var layerMenuX = 0, layerMenuY = 0, layerMenuResetX, layerMenuResetY;
var layerMenuWidth, layerMenuHeight;
var LayerButtonColor = "#E7F5A7", LayerButtonTextColor = "blue", LayerButtonOverColor = "#AAC52B", LayerButtonOverTextColor = "red";
var dragLayerScrollbar = false, scrollLayerBar = false, dragLayerMenu = false;

/***initLayer()****************************************************************
*** This initiates the layer menu                                           ***
******************************************************************************/
function initLayer() {
	layerMenu = svgdoc.getElementById('layerMenu');
	layerButtons = svgdoc.getElementById('layerButtons');
	layerScrollbar = svgdoc.getElementById('LayerScrollbarDrag');
	setLayerMenu();
	
	var mapkid = map.childNodes;
	var featurekid = feature.childNodes;
	for (var i=0;i<mapkid.length;i++)
		createLayerButton(mapkid.item(i),featurekid.item(i));
}
/***makeLayerEditable()********************************************************
*** Makes the layer editable and changes the button in the layermenu        ***
******************************************************************************/
function makeLayerEditable(mapLayer,layerName) {
	mapLayer.setAttribute('editable', "true")
	var featureLayer = svgdoc.getElementById("Features-" + layerName);
	featureLayer.setAttribute('editable', "true")
	var layerButton = svgdoc.getElementById("LayerButton-Map-" + layerName);
	layerButton.childNodes.item(4).setAttributeNS("http://www.w3.org/1999/xlink","xlink:href","pic/edit.png");		
	layerButton.setAttribute('editable', "true");		
}
/***makeLayerVisible()*********************************************************
*** Makes the layer visible and changes the button in the layermenu         ***
******************************************************************************/
function makeLayerVisible(mapLayer,layerName) {
	mapLayer.setAttribute('visibility', "visible")
	var layerButton = svgdoc.getElementById("LayerButton-Map-" + layerName);

	var boundedLayer = svgdoc.getElementById("Bounded-" + layerName);
	if (boundedLayer != null) boundedLayer.setAttribute('visibility', "visible")

	var layerButton = svgdoc.getElementById("LayerButton-Map-" + layerName);
	layerButton.childNodes.item(3).setAttributeNS("http://www.w3.org/1999/xlink","xlink:href","pic/eye.png");
	layerButton.setAttribute('mapvisibility', "visible");
}
/***layerOver()****************************************************************
*** Changes the color of the layer and if the layer has a boundingbox it    ***
*** will make this visible for the user. This function also does the        ***
*** opposite if the out is true.                                            ***
******************************************************************************/
function layerOver(evt,out) {
	if ((moving) || (draging)) return;
	
	var p = evt.getTarget().parentNode;
	if (out) {
		p.childNodes.item(0).getStyle().setProperty('fill', LayerButtonColor);
		p.childNodes.item(1).getStyle().setProperty('fill', LayerButtonTextColor);
				
		var boundedLayer = svgdoc.getElementById(p.getAttribute('bounded'));
		if (boundedLayer != null) boundedLayer.setAttribute('fill', "none")
	}
	else {
		p.childNodes.item(0).getStyle().setProperty('fill', LayerButtonOverColor);
		p.childNodes.item(1).getStyle().setProperty('fill', LayerButtonOverTextColor);	
		var boundedLayer = svgdoc.getElementById(p.getAttribute('bounded'));
		if (boundedLayer != null) boundedLayer.setAttribute('fill', "#EFEDD8")
	}
}
/***changeEye()****************************************************************
*** When the user presses the eye on the button. This is the function that  ***
*** makes the map visible or hides it. If it hides it it will also removes  ***
*** all the selected points on this layer                                   ***
******************************************************************************/
function changeEye(evt) {
	var p = evt.getTarget().parentNode;
	var mapLayer = svgdoc.getElementById(p.getAttribute('map'));
	if (p.getAttribute('mapvisibility') == "visible") {
		evt.getTarget().setAttributeNS("http://www.w3.org/1999/xlink","xlink:href","pic/noneye.png");
		mapLayer.setAttribute('visibility', "hidden")
		p.setAttribute('mapvisibility', "hidden")
		findAndRemovePointsWithThisLayer(p.getAttribute('map'));

		var boundedLayer = svgdoc.getElementById(p.getAttribute('bounded'));
		if (boundedLayer != null) boundedLayer.setAttribute('visibility', "hidden")
	}
	else {
		evt.getTarget().setAttributeNS("http://www.w3.org/1999/xlink","xlink:href","pic/eye.png");		
		mapLayer.setAttribute('visibility', "visible")
		p.setAttribute('mapvisibility', "visible")
		var boundedLayer = svgdoc.getElementById(p.getAttribute('bounded'));
		if (boundedLayer != null) boundedLayer.setAttribute('visibility', "visible")
	}
}
/***changeEye()****************************************************************
*** When the user presses the edit on the button. This is the fucntion that ***
*** makes the layer editable or not. If not editable it will also removes   ***
*** all the selected points on this layer                                   ***
******************************************************************************/
function changeEdit(evt) {
	var p = evt.getTarget().parentNode;
	var mapLayer = svgdoc.getElementById(p.getAttribute('map'));
	var editLayer = svgdoc.getElementById(p.getAttribute('edit'));
	if (p.getAttribute('editable') == "true") {
		evt.getTarget().setAttributeNS("http://www.w3.org/1999/xlink","xlink:href","pic/nonedit.png");
		mapLayer.setAttribute('editable', "false")
		p.setAttribute('editable', "false")
		findAndRemovePointsWithThisLayer(p.getAttribute('map'));
	}
	else {
		evt.getTarget().setAttributeNS("http://www.w3.org/1999/xlink","xlink:href","pic/edit.png");		
		mapLayer.setAttribute('editable', "true")
		p.setAttribute('editable', "true")
	}
}
/***createLayerButton()********************************************************
*** Creates the Layerbutton with all the information needed to do what it   ***
*** does                                                                    ***
******************************************************************************/
function createLayerButton(map,feature) {
	var newButton = svgdoc.createElement("g");
	var ButtonX, ButtonY, ButtonWidth, ButtonHeight;
	var text = map.getAttribute('name');
	
	newButton.setAttribute('id', 'LayerButton-' + map.getAttribute('id'));
	newButton.setAttribute('feature', feature.getAttribute('id'));
	newButton.setAttribute('map', map.getAttribute('id'));
	newButton.setAttribute('bounded', 'Bounded-' + feature.getAttribute('layer'));
	newButton.setAttribute('editable', map.getAttribute('editable'));
	newButton.setAttribute('mapvisibility', map.getAttribute('visibility'));
	ButtonWidth = 105;
	ButtonHeigth = 100/4;
	ButtonX = 0.00;
	ButtonY = (layerLength*ButtonHeigth);
	var newRect = svgdoc.createElement("rect");
	newRect.setAttribute("id","LayerButton" + layerLength);
	newRect.setAttribute("x",ButtonX);
	newRect.setAttribute("y",ButtonY);
	newRect.setAttribute("width",ButtonWidth);
	newRect.setAttribute("height",ButtonHeigth);
	
	var newRectStyle = newRect.getStyle();
	newRectStyle.setProperty('stroke','white');
	newRectStyle.setProperty('fill',LayerButtonColor);
	newRectStyle.setProperty('fill-opacity','0.7');
	
	newButton.appendChild(newRect);
	
	var newText = svgdoc.createElement("text");
	newText.appendChild(svgdoc.createTextNode (text));
	newText.setAttribute("id","LayerButtonText" + layerLength);
	newText.setAttribute("x",(ButtonX+20));
	newText.setAttribute("y",(ButtonY+18));
	var textStyle = newText.getStyle();
	textStyle.setProperty("fill",LayerButtonTextColor);
	newButton.appendChild(newText);

	var newRectOnTop = newRect.cloneNode(true);
	newRectOnTop.setAttribute("position",layerLength);
	newRectOnTop.setAttribute("id","LayerButtonOnTop" + layerLength);
	newRectOnTop.setAttribute("onmouseover","layerOver(evt,false)");
	newRectOnTop.setAttribute("onmouseout","layerOver(evt,true)");
	var newRectOnTopStyle = newRectOnTop.getStyle();
	newRectOnTopStyle.setProperty('fill-opacity',0.0);
	
	newButton.appendChild(newRectOnTop);
	
	var newImageEye = svgdoc.createElement("image");

	if (map.getAttribute('visibility') == "visible")
		newImageEye.setAttributeNS("http://www.w3.org/1999/xlink","xlink:href","pic/eye.png");
	else
		newImageEye.setAttributeNS("http://www.w3.org/1999/xlink","xlink:href","pic/noneye.png");		

	newImageEye.setAttribute("width", (12*0.8));
	newImageEye.setAttribute("height", (10*0.8));
	newImageEye.setAttribute("x",ButtonX+5);
	newImageEye.setAttribute("y",ButtonY+5);
	newImageEye.setAttribute("layer",ButtonY+5);
	newImageEye.setAttribute("onclick","changeEye(evt)");
	newImageEye.setAttribute("onmouseover","layerOver(evt,false)");
	newImageEye.setAttribute("onmouseout","layerOver(evt,true)");

	newButton.appendChild(newImageEye);
	
	var newImageEdit = svgdoc.createElement("image");

	if (map.getAttribute('editable') == "true")
		newImageEdit.setAttributeNS("http://www.w3.org/1999/xlink","xlink:href","pic/edit.png");
	else
		newImageEdit.setAttributeNS("http://www.w3.org/1999/xlink","xlink:href","pic/nonedit.png");		

	newImageEdit.setAttribute("width", (12*0.8));
	newImageEdit.setAttribute("height", (10*0.8));
	newImageEdit.setAttribute("x",ButtonX+5);
	newImageEdit.setAttribute("y",ButtonY+15);
	newImageEdit.setAttribute("onclick","changeEdit( evt )");
	newImageEdit.setAttribute("onmouseover","layerOver(evt,false)");
	newImageEdit.setAttribute("onmouseout","layerOver(evt,true)");

	newButton.appendChild(newImageEdit);

	layerButtons.appendChild(newButton);
	layerLength ++;
	var newtransform = "translate('0,0')";
	layerButtons.setAttribute('transform',newtransform);
	moveAndResizeLayerScrollbar();
}
/***goToLayerScrollPosition()**************************************************
*** This is if the user clicks on the scollbar back this will               ***
*** find out on what side of the scrollbar and go to left or right          ***
******************************************************************************/
function goToLayerScrollPosition(evt) {
	if (evt.getClientY() > (layerScrollbarY+layerMenuY))
		moveLayerScrollBar(50,true);
	else
		moveLayerScrollBar(-50,true);
}
/***moveLayerMenu()************************************************************
*** A function to move the menu. The parameter shows how much it will move  ***
*** It also stores what the x and y value for the menu at all time. And if  ***
*** the windowscale is wider then the 800/500 scale it will add to the X so ***
*** much that it would look like it goes up to the right corner.            ***
***                                                                         ***
*** View the documentation for more information about this                  ***
******************************************************************************/
function moveLayerMenu(moveX, moveY) {
	layerMenuX += moveX;
	layerMenuY += moveY;

	if (windowscaleway) {
		var newWidth = (130 * windowscale);
		var newXAdd = (130 * windowscaleWidth);
		newXAdd -= newWidth;
	}
	else
		var newXAdd = 0;

	layerMenu.setAttribute('x',layerMenuX+newXAdd + "");
	layerMenu.setAttribute('y',layerMenuY + "");
}
/***setLayerMenu()*************************************************************
*** This finds out what the x and y value is from the start and stores that ***
*** in case it will ever be reset. After that it will scale to fit the      ***
*** current view that the SVG window is in                                  ***
******************************************************************************/
function setLayerMenu() {	
	layerMenuResetX = parseInt(layerMenu.getAttribute('x'));
	layerMenuResetY = parseInt(layerMenu.getAttribute('y'));	
	layerMenuX = (layerMenuResetX * ratioWidth);
	layerMenuY = (layerMenuResetY * ratioHeight);		
	
	scaleLayerMenu(false);
}
/***scaleLayerMenu()***********************************************************
*** This scales the menu so it will be put at the right place with the      ***
*** right height and width                                                  ***
******************************************************************************/
function scaleLayerMenu(move) {	
	var newWidth = (130 * windowscale);
	var newHeight = (130 * windowscale);	
	layerMenu.setAttribute('width', newWidth + "");
	layerMenu.setAttribute('height', newHeight + "");	

	if (move) moveLayerMenu(((layerMenuX*ratioWidth) - layerMenuX), ((layerMenuY*ratioHeight) - layerMenuY));
}
/***startClickLayer()**********************************************************
*** cheks if it is inside the layermenu that is been clicked                ***
******************************************************************************/
function startClickLayer(evt) {
	var target = evt.getTarget().getAttribute('id');
	if (target == 'LayerScrollbarDrag')
		dragLayerScrollbar = true;
	if (target == 'LayerMenuBackground')
		dragLayerMenu = true;	
}
/***stopClickLayer()**********************************************************
*** cheks if it is inside the layer menu that has stopped clicked.         ***
*****************************************************************************/
function stopClickLayer(evt) {
	if (evt.getTarget().getAttribute('id') == 'LayerScrollbarDrag')
		if (dragLayerScrollbar) {
			dragLayerScrollbar = false;
			return;
		}
	if (evt.getTarget().getAttribute('id') == 'LayerMenuBackground')
		if (dragLayerMenu) {
			dragLayerMenu = false;
			return;
		}
}
/***checkMouseLayer()**********************************************************
*** if the menu is being dragged or the layerwindow is beeing dragged       ***
*** this is the function that moves this                                    ***
******************************************************************************/
function checkMouseLayer(evt) {
	if (dragLayerScrollbar) {
		var target = evt.getTarget().getAttribute('id');
		if ((target != 'LayerScrollbarDrag') && (target != 'LayerScrollbarBack'))
			dragLayerScrollbar = false;
		else {
			if ((currentClientY - lastClientY) != 0)
				moveLayerScrollBar((currentClientY-lastClientY),false);
		}
	}
	if (dragLayerMenu) 
		moveLayerMenu((currentClientX-lastClientX),(currentClientY-lastClientY));	
}
/***resetHistoryMenu()*********************************************************
*** reset the menu to it's orignial view                                    ***
******************************************************************************/
function resetLayerMenu() {
	layerMenu.setAttribute('x',(layerMenuResetX * ratioWidth) + "");
	layerMenu.setAttribute('y',(layerMenuResetY * ratioHeight) + "");
}
/***moveLayerScrollBar()*******************************************************
*** Moves the layermenus scrollbar the way the user is pushing it           ***
*** if the bol attribute is true it will move in percent. Not in pixsels    ***
******************************************************************************/
function moveLayerScrollBar(move,bol) {
	if (layerLength < 5) return;
	var movePercent = 100/(layerLength*(100/4));
	if (bol) move *= movePercent;
	layerScrollbarY += move;
	
	if (layerScrollbarY > (103-layerScrollbarHeight))
		layerScrollbarY = (103-layerScrollbarHeight);
	if (layerScrollbarY < 33)
		layerScrollbarY = 33;
	
	layerScrollbar.setAttribute('y',layerScrollbarY + "");
	
	var layerButtonsTranslateY = -(100/layerScrollbarHeight) * (layerScrollbarY-33);
	var newtransform = "translate('0," + layerButtonsTranslateY + "')";	
	layerButtons.setAttribute('transform',newtransform);
}
/***moveAndResizeLayerScrollbar()**********************************************
*** Moves the historymenu to the last posistion and resizes the scrollbar   ***
*** to the new size                                                         ***
******************************************************************************/
function moveAndResizeLayerScrollbar() {
	layerScrollbarHeight = 74 * (100/(layerLength*(100/4)));
	layerScrollbarY = 33;
	if (layerScrollbarHeight < 74)
		layerScrollbar.getStyle().setProperty('visibility','visible');
	else
		layerScrollbar.getStyle().setProperty('visibility','hidden');
	layerScrollbar.setAttribute('y',layerScrollbarY + "");
	layerScrollbar.setAttribute('height',layerScrollbarHeight + "");
}
